using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using System.Web;
using Vevo.Shared.Domain.SystemServices;

namespace Vevo.Shared.SystemServices
{
    [Serializable]
    public class FileManager : IFileManager
    {
        #region Private

        private string[] ConvertToTildeRelativePaths( string[] fullPaths )
        {
            string appLocalPath = HttpContext.Current.Server.MapPath( "~/" );

            StringCollection collection = new StringCollection();
            foreach (string fullPath in fullPaths)
            {
                collection.Add( ConvertToTildeRelativePath( appLocalPath, fullPath ) );
            }

            // Copy to string[]
            string[] result = new string[collection.Count];
            collection.CopyTo( result, 0 );
            return result;
        }

        private string ConvertToTildeRelativePath( string appLocalPath, string fullPath )
        {
            if (fullPath.ToLower().StartsWith( appLocalPath.ToLower() ))
            {
                return "~/" + fullPath.Substring( appLocalPath.Length ).Replace( '\\', '/' );
            }
            else
            {
                return fullPath;
            }
        }

        #endregion


        #region IFileManager Members

        public void Move( string sourceFileName, string destFileName )
        {
            // Delete destination file if already existed
            if (File.Exists( GetFullPath( destFileName ) ))
                File.Delete( GetFullPath( destFileName ) );

            File.Move( GetFullPath( sourceFileName ), GetFullPath( destFileName ) );
        }

        public void Copy( string sourceFileName, string destFileName )
        {
            File.Copy( GetFullPath( sourceFileName ), GetFullPath( destFileName ) );
        }

        public void Delete( string filePath )
        {
            File.Delete( GetFullPath( filePath ) );
        }

        public bool Exists( string filePath )
        {
            return File.Exists( GetFullPath( filePath ) );
        }

        public bool ContainsUnicodeBom( string filePath )
        {
            bool unicode = false;
            Stream file = null;
            try
            {
                file = CreateStream( filePath, FileMode.Open, FileAccess.Read, FileShare.Read );

                if (file.CanSeek)
                {
                    byte[] bom = new byte[4]; // Get the byte-order mark, if there is one
                    file.Read( bom, 0, 4 );
                    if ((bom[0] == 0xef && bom[1] == 0xbb && bom[2] == 0xbf) || // utf-8
                        (bom[0] == 0xff && bom[1] == 0xfe) || // ucs-2le, ucs-4le, and ucs-16le
                        (bom[0] == 0xfe && bom[1] == 0xff) || // utf-16 and ucs-2
                        (bom[0] == 0 && bom[1] == 0 && bom[2] == 0xfe && bom[3] == 0xff)) // ucs-4
                    {
                        unicode = true;
                    }
                }
            }
            finally
            {
                if (file != null)
                    file.Close();
            }

            return unicode;
        }

        public long GetFileSize( string path )
        {
            FileInfo fileInfo = new FileInfo( GetFullPath( path ) );
            return fileInfo.Length;
        }

        public DateTime GetFileModifiedDate( string path )
        {
            FileInfo fileInfo = new FileInfo( GetFullPath( path ) );
            return fileInfo.LastWriteTime;
        }

        public string GetFullPath( string path )
        {
            return HttpContext.Current.Server.MapPath( path.Trim() );
        }

        public string ReadToEnd( string filePath )
        {
            string data;
            using (StreamReader streamReader =
                new StreamReader( HttpContext.Current.Server.MapPath( filePath )))
            {
                data = streamReader.ReadToEnd();
            }
            return data;
        }

        public void SaveAll( string sourceFileName, string content )
        {
            File.WriteAllText( GetFullPath( sourceFileName ), content );
        }

        public Stream CreateStream( string filePath, FileMode mode )
        {
            return new FileStream( GetFullPath( filePath ), mode );
        }

        public Stream CreateStream( string filePath, FileMode mode, FileAccess access )
        {
            return new FileStream( GetFullPath( filePath ), mode, access );
        }

        public Stream CreateStream( string filePath, FileMode mode, FileAccess access, FileShare share )
        {
            return new FileStream( GetFullPath( filePath ), mode, access, share );
        }

        public TextReader CreateTextReader( string filePath )
        {
            return new StreamReader( GetFullPath( filePath ) );
        }

        public TextReader CreateTextReader( string filePath, Encoding encoding )
        {
            return new StreamReader( GetFullPath( filePath ), encoding );
        }

        public TextWriter CreateTextWriter( string filePath, bool append )
        {
            return new StreamWriter( GetFullPath( filePath ), append );
        }

        public TextWriter CreateTextWriter( string filePath, bool append, Encoding encoding )
        {
            return new StreamWriter( GetFullPath( filePath ), append, encoding );
        }

        public string[] SearchFiles( string path )
        {
            return SearchFiles( path, "*" );
        }

        public string[] SearchFiles( string path, string searchPattern )
        {
            return SearchFiles( path, searchPattern, false );
        }

        public string[] SearchFiles( string path, string searchPattern, bool includeSubDirectories )
        {
            SearchOption option;
            if (includeSubDirectories)
                option = SearchOption.AllDirectories;
            else
                option = SearchOption.TopDirectoryOnly;

            string[] foundPaths = Directory.GetFiles( GetFullPath( path ), searchPattern, option );
            return ConvertToTildeRelativePaths( foundPaths );
        }

        #endregion
    }
}
