﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Shared.DataAccess;
using Vevo.Domain;
using Vevo.Base.Data;

namespace Vevo.Data.Implementation
{
    internal class BlogCategoryItemAccess
    {
        #region Private

        private IList<String> ConvertToBlogCategoryIDs( DataTable table )
        {
            IList<String> list = new List<String>();
            foreach (DataRow row in table.Rows)
            {
                list.Add( row["BlogCategoryID"].ToString() );
            }

            return list;
        }

        private void Create( string blogID, IList<String> blogCategoryIDs, Dictionary<string, string> blogCategoryList )
        {
            string sortOrder = String.Empty;
            foreach (string blogCategoryID in blogCategoryIDs)
            {
                int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM BlogCategoryItem;" ) );

                int result = DataAccess.ExecuteNonQuery(
                    "INSERT INTO BlogCategoryItem (BlogID, BlogCategoryID, SortOrder) " +
                    "VALUES (@BlogID, @BlogCategoryID, @SortOrder); ",
                DataAccess.CreateParameterString( blogID ),
                DataAccess.CreateParameterString( blogCategoryID ),
                DataAccess.CreateParameterInt32( recCount++ ) );

                if (result != 1)
                {
                    Exception exception = new ApplicationException(
                            "Failed creating product. " +
                        "Inserting into ProductCategory returns " + result.ToString() );
                    DataAccessHelper.LogError( exception );
                    throw exception;
                }
            }
        }

        private Dictionary<string, string> GetBlogCategoryItemSortOrder( string blogID, IList<String> blogCategoryIDs )
        {
            Dictionary<string, string> blogCategoryList = new Dictionary<string, string>();
            DataTable table = DataAccess.ExecuteSelect(
                  "SELECT * " +
                  "FROM BlogCategoryItem " +
                  "WHERE BlogID = @BlogID; ",
                  DataAccess.CreateParameterString( blogID ) );
            foreach (DataRow row in table.Rows)
            {
                if (blogCategoryIDs.Contains( row["BlogCategoryID"].ToString() ))
                {
                    if (String.IsNullOrEmpty( row["BlogCategoryID"].ToString() ))
                    {
                        row["BlogCategoryID"] = "0";
                    }
                    blogCategoryList.Add( row["BlogCategoryID"].ToString(), row["SortOrder"].ToString() );
                }
            }
            return blogCategoryList;
        }

        private string CreateSortByText( string sortBy )
        {
            if (String.IsNullOrEmpty( sortBy.Trim() ))
                return " ";
            else
                return " ORDER BY " + sortBy;
        }

        #endregion

        #region Public Methods

        public void SaveAll( string blogID, IList<String> blogCategoryIDs )
        {
            Dictionary<string, string> blogCategoryList = GetBlogCategoryItemSortOrder( blogID, blogCategoryIDs );
            DeleteByBlogID( blogID );
            Create( blogID, blogCategoryIDs, blogCategoryList );
        }

        public void DeleteByBlogID( string blogID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM BlogCategoryItem " +
                "WHERE BlogID = @BlogID; ",
                DataAccess.CreateParameterString( blogID ) );
        }

        public IList<String> GetBlogCategoryIDByBlogID( string blogID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT BlogCategoryID " +
                "FROM BlogCategoryItem " +
                "WHERE BlogID = @BlogID; ",
                DataAccess.CreateParameterString( blogID ) );

            return ConvertToBlogCategoryIDs( table );
        }

        public DataTable GetBlogsByCategoryID( string sortBy, string categoryID, string storeID, BoolFilter isEnabled )
        {
            return DataAccess.ExecuteSelect(
                "SELECT DISTINCT Blog.BlogID, BlogTitle, UrlName, IsEnabled, ShortContent, BlogContent, Tags, " +
                "MetaTitle, MetaKeyword, MetaDescription, Publisher, CreateDate, ImageFile " +
                "FROM BlogCategoryItem " +
                "    RIGHT JOIN Blog ON BlogCategoryItem.BlogID = Blog.BlogID " +
                "    LEFT JOIN BlogStore ON Blog.BlogID = BlogStore.BlogID " +
                "WHERE BlogCategoryID = @BlogCategoryID AND StoreID = @StoreID AND CreateDate < GETDATE()" +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ),
                DataAccess.CreateParameterString( categoryID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( isEnabled.ToString() ) );
        }

        public int GetBlogCountByCategoryID( string categoryID, string storeID, BoolFilter isEnabled )
        {
            string blogCount = DataAccess.ExecuteScalar(
                "SELECT DISTINCT COUNT (Blog.BlogID) AS [RowCount] " +
                "FROM BlogCategoryItem " +
                "    RIGHT JOIN Blog ON BlogCategoryItem.BlogID = Blog.BlogID " +
                "    LEFT JOIN BlogStore ON Blog.BlogID = BlogStore.BlogID " +
                "WHERE BlogCategoryID = @BlogCategoryID AND StoreID = @StoreID AND CreateDate < GETDATE() " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ),
                DataAccess.CreateParameterString( categoryID ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterString( isEnabled.ToString() ) );

            return Vevo.Shared.Utilities.ConvertUtilities.ToInt32( blogCount );
        }

        #endregion
    }
}
