using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;

namespace Vevo.Data.Implementation
{
    internal class ProductStockAccess
    {
        #region Private

        private IList<String> GetOptionItemIDs( string optionCombinationID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ProductOptionItemID " +
                "FROM OptionCombination " +
                "WHERE OptionCombinationID = @OptionCombinationID;",
                DataAccess.CreateParameterString( optionCombinationID ) );

            return RepositoryUtilities.ConvertTableToStringList( table, "ProductOptionItemID" );
        }

        private ProductStock CreateOneFromRow( DataRow row, IList<String> optionItemIDs )
        {
            ProductStock productStock = new ProductStock( optionItemIDs );
            productStock.OptionCombinationID = row["OptionCombinationID"].ToString();
            productStock.Stock = ConvertUtilities.ToInt32( row["Stock"] );
            productStock.StockEnabled = ConvertUtilities.ToBoolean( row["StockEnabled"] );

            return productStock;
        }

        public void InsertOptionCombinationIDs(
            string optionCombinationID,
            IList<String> optionItemIDs )
        {
            foreach (string optionItemID in optionItemIDs)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO OptionCombination ( OptionCombinationID, ProductOptionItemID ) " +
                    "VALUES ( @OptionCombinationID, @OptionItemID );",
                    DataAccess.CreateParameterString( optionCombinationID ),
                    DataAccess.CreateParameterString( optionItemID ) );
            }
        }

        private void SaveStockNoOption( string productID, ProductStock productStock )
        {
            if (Update( productID, productStock ) == 0)
                InsertProductStock( productID, "0", productStock );
        }

        private void InsertProductStock(
            string productID,
            string optionCombinationID,
            ProductStock productStock )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO ProductStock ( ProductID, Stock, OptionCombinationID, StockEnabled ) " +
                "VALUES ( @ProductID, @Stock, @OptionCombinationID, @StockEnabled )",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterInt32( productStock.Stock ),
                DataAccess.CreateParameterString( optionCombinationID ),
                DataAccess.CreateParameterBool( productStock.StockEnabled ) );
        }

        private string Create( string productID, ProductStock productStock )
        {
            string combinationIDMax = GetCombinationIDMax();

            InsertOptionCombinationIDs( combinationIDMax, productStock.OptionItemIDs );

            InsertProductStock( productID, combinationIDMax, productStock );
            productStock.OptionCombinationID = combinationIDMax;

            return combinationIDMax;
        }

        private int Update( string productID, ProductStock productStock )
        {
            return DataAccess.ExecuteNonQuery(
                "UPDATE ProductStock " +
                "SET Stock = @Stock, " +
                "    StockEnabled = @StockEnabled " +
                "WHERE ProductID = @ProductID " +
                "AND OptionCombinationID = @OptionCombinationID",
                DataAccess.CreateParameterInt32( productStock.Stock ),
                DataAccess.CreateParameterBool( productStock.StockEnabled ),
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( productStock.OptionCombinationID ) );
        }

        private string GetCombinationIDMax()
        {
            string combinationID = DataAccess.ExecuteScalar(
                "SELECT MAX( OptionCombinationID ) + 1 FROM OptionCombination; " );

            if (String.IsNullOrEmpty( combinationID ))
                return "1";
            else
                return combinationID;
        }

        #endregion


        #region Public Methods

        public void SaveAll( string productID, IList<ProductStock> productStocks )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DeleteByProductID( productID );
                InsertProductStock(productID, productStocks);

                transaction.Complete();
            }
        }

        public void InsertProductStock(string productID, IList<ProductStock> productStocks)
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                foreach (ProductStock productStock in productStocks)
                {
                    if (ConvertUtilities.ToInt32(productStock.OptionCombinationID) == 0 &&
                        productStock.OptionItemIDs.Length == 0)
                    {
                        SaveStockNoOption(productID, productStock);
                    }
                    else
                    {
                        Create(productID, productStock);
                    }

                    //else if (ConvertUtilities.ToInt32( productStock.OptionCombinationID ) == 0)
                    //{
                    //    Create( productID, productStock );
                    //}
                    //else
                    //{
                    //    Update( productID, productStock );
                    //}
                }
                transaction.Complete();
            }
        }

        public void DeleteByProductID( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OptionCombinationID FROM ProductStock " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( productID ) );

            foreach (DataRow row in table.Rows)
            {
                DataAccess.ExecuteNonQuery(
                    "DELETE FROM OptionCombination " +
                    "WHERE OptionCombinationID = @OptionCombinationID",
                    DataAccess.CreateParameterString( row["OptionCombinationID"].ToString() ) );
            }

            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductStock " +
                "WHERE ProductID = @ProductID",
                DataAccess.CreateParameterString( productID ) );
        }

        public IList<ProductStock> GetByProductID( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ProductID, Stock, OptionCombinationID, StockEnabled " +
                "FROM ProductStock " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( productID ) );

            IList<ProductStock> list = new List<ProductStock>();
            foreach (DataRow row in table.Rows)
            {
                IList<String> optionItemIDs = GetOptionItemIDs( row["OptionCombinationID"].ToString() );
                list.Add( CreateOneFromRow( row, optionItemIDs ) );
            }

            return list;
        }

        #endregion

    }
}
