﻿using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class SpecificationItemValueRepositoryNoCache : ISpecificationItemValueRepository
    {
        #region Private

        private const string AllSpecificationItemValueLocaleFields = " SpecificationItemValueID , CultureID, DisplayValue ";
        private const string AllSpecificationItemValueFields =
           " SpecificationItemValue.SpecificationItemValueID, SpecificationItemValue.SpecificationItemID, Value, DisplayValue ";

        private SpecificationItemValue CreateOneFromRow( Culture culture, DataRow row )
        {
            SpecificationItemValue specificationItemValue = new SpecificationItemValue( culture );

            specificationItemValue.SpecificationItemValueID = row["SpecificationItemValueID"].ToString();
            specificationItemValue.SpecificationItemID = row["SpecificationItemID"].ToString();
            specificationItemValue.Value = row["Value"].ToString();
            specificationItemValue.DisplayValue = row["DisplayValue"].ToString();

            return specificationItemValue;
        }

        private SpecificationItemValueLocale CreateOneLocaleFromRow( DataRow row )
        {
            SpecificationItemValueLocale locale = new SpecificationItemValueLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.DisplayValue = row["DisplayValue"].ToString();

            return locale;
        }

        private void CopySpecificationItemLocales( SpecificationItemValue specificationItemValue, IList<ILocale> locales )
        {
            foreach (SpecificationItemValueLocale locale in locales)
            {
                specificationItemValue.Locales.Add( locale );
            }
        }

        private IList<SpecificationItemValue> ConvertSpecificationItemTableToList( Culture culture, DataTable table )
        {
            IList<SpecificationItemValue> list = new List<SpecificationItemValue>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneFromRow( culture, row ) );

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "specificationitemvalueid":
                    return "SpecificationItemValue.SpecificationItemValueID";
                default:
                    return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "specificationitemvalueid" ))
                return "ORDER BY SpecificationItemValue." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private IList<SpecificationItemValue> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<SpecificationItemValue> list = new List<SpecificationItemValue>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private void DeleteLocales( string specificationItemValueID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM SpecificationItemValueLocale " +
                "WHERE SpecificationItemValueID = @SpecificationItemValueID; ",
                DataAccess.CreateParameterString( specificationItemValueID ) );
        }

        private void InsertLocales( SpecificationItemValue specificationItemValue )
        {
            foreach (SpecificationItemValueLocale locale in specificationItemValue.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO SpecificationItemValueLocale ( SpecificationItemValueID, CultureID, DisplayValue ) " +
                    "VALUES ( @SpecificationItemValueID, @CultureID, @DisplayValue )",
                    DataAccess.CreateParameterString( specificationItemValue.SpecificationItemValueID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.DisplayValue ) );
            }
        }

        private SpecificationItemValue Create( SpecificationItemValue specificationItemValue )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO SpecificationItemValue ( SpecificationItemID, Value ) " +
                "VALUES ( @SpecificationItemID, @Value );" +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( specificationItemValue.SpecificationItemID ),
                DataAccess.CreateParameterString( specificationItemValue.Value ) );

            specificationItemValue.SpecificationItemValueID = newID;

            InsertLocales( specificationItemValue );

            return specificationItemValue;
        }

        private SpecificationItemValue Update( SpecificationItemValue specificationItem )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE SpecificationItemValue " +
                "SET SpecificationItemID = @SpecificationItemID, Value = @Value " +
                "WHERE SpecificationItemValueID = @SpecificationItemValueID ;",
                DataAccess.CreateParameterString( specificationItem.SpecificationItemID ),
                DataAccess.CreateParameterString( specificationItem.Value ),
                DataAccess.CreateParameterString( specificationItem.SpecificationItemValueID ) );

            DeleteLocales( specificationItem.SpecificationItemValueID );
            InsertLocales( specificationItem );

            return specificationItem;
        }
        #endregion

        #region Public

        public SpecificationItemValue Save( SpecificationItemValue specificationItemValue )
        {
            SpecificationItemValue result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (specificationItemValue.SpecificationItemValueID == "0")
                    result = Create( specificationItemValue );
                else
                    result = Update( specificationItemValue );

                transaction.Complete();
            }

            return result;
        }
        public IList<ILocale> GetLocales( string specificationItemValueID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllSpecificationItemValueLocaleFields +
                "FROM SpecificationItemValueLocale " +
                "WHERE SpecificationItemValueID  = @SpecificationItemValueID ; ",
                DataAccess.CreateParameterString( specificationItemValueID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SpecificationItemValue.SpecificationItemValueID, SpecificationItemID, Value, DisplayValue " +
                "FROM SpecificationItemValue LEFT JOIN " +
                "(SELECT * FROM SpecificationItemValueLocale) AS L " +
                "ON SpecificationItemValue.SpecificationItemValueID = L.SpecificationItemValueID " +
                "WHERE 1 = 0 ;" );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }


        public void Delete( string specificationItemValueID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                 "DELETE FROM SpecificationItemValue WHERE SpecificationItemValueID = @SpecificationItemValueID;",
                DataAccess.CreateParameterString( specificationItemValueID ) );
                DeleteLocales( specificationItemValueID );

                transaction.Complete();
            }
        }

        public void DeleteBySpecificationItemID( string specificationItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT * FROM SpecificationItemValue WHERE SpecificationItemID = @SpecificationItemID;",
               DataAccess.CreateParameterString( specificationItemID ) );

            foreach (DataRow row in table.Rows)
            {
                Delete( row["SpecificationItemValueID"].ToString() );
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM SpecificationItemValueLocale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public DataTable GetAll( Culture culture )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllSpecificationItemValueFields +
                "FROM SpecificationItemValue " +
                "   LEFT JOIN (SELECT * FROM SpecificationItemValueLocale WHERE CultureID = @CultureID) AS ValueLocale " +
                "   ON SpecificationItemValue.SpecificationItemValueID = ValueLocale.SpecificationItemValueID ",
                DataAccess.CreateParameterString( culture.CultureID ) );

            return table;
        }

        public SpecificationItemValue GetOne( Culture culture, string specificationItemValueID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllSpecificationItemValueFields +
                "FROM SpecificationItemValue " +
                "   LEFT JOIN (SELECT * FROM SpecificationItemValueLocale WHERE CultureID = @CultureID) AS ValueLocale " +
                "   ON SpecificationItemValue.SpecificationItemValueID = ValueLocale.SpecificationItemValueID " +
                "WHERE SpecificationItemValue.SpecificationItemValueID = @SpecificationItemValueID; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( specificationItemValueID ) );

            if (table.Rows.Count > 0)
            {
                SpecificationItemValue specificationItemValue = CreateOneFromRow( culture, table.Rows[0] );
                CopySpecificationItemLocales( specificationItemValue, GetLocales( specificationItemValueID ) );
                return specificationItemValue;
            }
            else
                return SpecificationItemValue.Null;
        }

        public SpecificationItemValue GetOneBySpecItemIDAndValue( Culture culture, string specificationItemID, string value )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllSpecificationItemValueFields +
                "FROM SpecificationItemValue " +
                "   LEFT JOIN (SELECT * FROM SpecificationItemValueLocale WHERE CultureID = @CultureID) AS ValueLocale " +
                "   ON SpecificationItemValue.SpecificationItemValueID = ValueLocale.SpecificationItemValueID " +
                "WHERE SpecificationItemValue.SpecificationItemID = @SpecificationItemID and Value = @Value; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( specificationItemID ),
                DataAccess.CreateParameterString( value ) );

            if (table.Rows.Count > 0)
            {
                SpecificationItemValue specificationItemValue = CreateOneFromRow( culture, table.Rows[0] );
                CopySpecificationItemLocales( specificationItemValue, GetLocales( specificationItemValue.SpecificationItemValueID ) );
                return specificationItemValue;
            }
            else
                return SpecificationItemValue.Null;
        }

        public SpecificationItemValue GetOneByValue( Culture culture, string value )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllSpecificationItemValueFields +
                "FROM SpecificationItemValue " +
                "   LEFT JOIN (SELECT * FROM SpecificationItemValueLocale WHERE CultureID = @CultureID) AS ValueLocale " +
                "   ON SpecificationItemValue.SpecificationItemValueID = ValueLocale.SpecificationItemValueID " +
                "WHERE SpecificationItemValue.Value = @Value; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( value ) );

            if (table.Rows.Count > 0)
            {
                SpecificationItemValue specificationItemValue = CreateOneFromRow( culture, table.Rows[0] );
                CopySpecificationItemLocales( specificationItemValue, GetLocales( specificationItemValue.SpecificationItemValueID ) );
                return specificationItemValue;
            }
            else
                return SpecificationItemValue.Null;
        }

        public IList<SpecificationItemValue> GetBySpecificationItemID( Culture culture, string specificationItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllSpecificationItemValueFields +
                "FROM SpecificationItemValue " +
                "   LEFT JOIN (SELECT * FROM SpecificationItemValueLocale WHERE CultureID = @CultureID) AS ValueLocale " +
                "   ON SpecificationItemValue.SpecificationItemValueID = ValueLocale.SpecificationItemValueID " +
                "WHERE SpecificationItemValue.SpecificationItemID = @SpecificationItemID; ",
               DataAccess.CreateParameterString( culture.CultureID ),
               DataAccess.CreateParameterString( specificationItemID ) );

            return ConvertSpecificationItemTableToList( culture, table );
        }

        public IList<SpecificationItemValue> SearchSpecificationItemValue(
            Culture culture,
            string specificationItemID,
            string sortBy,
            SearchFilter searchfilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            string sqlFieldName = ConvertFieldName( searchfilter.FieldName );
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchfilter, sqlFieldName, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( specificationItemID ) );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM (" +
                "SELECT " + AllSpecificationItemValueFields +
                "FROM SpecificationItemValue " +
                "   LEFT JOIN (SELECT * FROM SpecificationItemValueLocale WHERE CultureID = @CultureID) AS ValueLocale " +
                "   ON SpecificationItemValue.SpecificationItemValueID = ValueLocale.SpecificationItemValueID " +
                commandSearch +
                "AND SpecificationItemID = @SpecificationItemID " +
                ") AS SpecificationItemValue ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
            DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            //DataColumn[] keys = new DataColumn[] { table.Columns["OptionItem.OptionItemID"] };
            //table.PrimaryKey = keys;
            return ConvertTableToList( culture, table, startIndex, endIndex );
        }

        public bool IsSpecificationItemValueExist(string value, string specificationItemID)
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SpecificationItemValueID FROM SpecificationItemValue " +
                "WHERE SpecificationItemID = @ID " +
                "AND Value = @Value; ",
                DataAccess.CreateParameterString(specificationItemID),
                DataAccess.CreateParameterString(value));

            if (table.Rows.Count > 0)
            {
                return true;
            }
            return false;
        }

        #endregion
    }
}
