using System;
using System.Xml;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;
using Vevo.PaymentSignatureLib;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Payments
{
    public class PaymentAppGateway
    {
        #region Private

        private CheckoutDetails _checkoutDetails;


        private string XMLAuthenticationBuilder()
        {
            PaymentOption paymentOption = DataAccessContext.PaymentOptionRepository.GetOne(
                Culture.Null, _checkoutDetails.PaymentMethod.Name );

            string[] paymentConfigList = paymentOption.AuthenticationConfigs.Split(
                new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries );

            string xmlData = "   <Authentication>";

            for (int i = 0; i < paymentConfigList.Length; i++)
            {
                string configName = paymentConfigList[i].Trim();
                if (!String.IsNullOrEmpty( configName ))
                {
                    xmlData +=
                        "       <Item>" +
                        "          <Name>" + configName + "</Name>" +
                        "          <Value>" + DataAccessContext.Configurations.GetValue( configName ) + "</Value>" +
                        "       </Item>";
                }
            }

            xmlData += "   </Authentication>";

            return xmlData;
        }

        private string XMLShoppingCartBuilder( Culture culture, Currency currency, Cart shoppingCart, WholesaleStatus wholesaleStatus )
        {
            string xmlData = "  <ShoppingCart>";

            foreach (ICartItem item in shoppingCart.CartItems)
            {
                xmlData += "    <CartItem>" +
                    "               <ProductID>" + item.ProductID + "</ProductID>" +
                    "               <Quantity> " + item.Quantity + "</Quantity>" +
                    "               <Name> " + EncodeSpecialXmlCharacters( item.GetName( culture, currency ).Replace( "<br>", "," ) ) + "</Name>" +
                    "               <Price> " + item.GetCheckoutUnitPrice( wholesaleStatus ) + "</Price>" +
                    "               <Tangible> " + !item.IsDownloadable + "</Tangible>" +
                    "           </CartItem>";
            }

            xmlData += "    </ShoppingCart>";

            return xmlData;
        }

        private PaymentAppResult GeneratePaymentResult( XmlDocument xml )
        {
            PaymentAppResult paymentResult = new PaymentAppResult(
                EnumUtilities<PaymentAppResult.PaymentStatus>.Parse( xml.GetElementsByTagName( "Status" )[0].InnerText ),
                xml.GetElementsByTagName( "GatewayOrderID" )[0].InnerText,
                xml.GetElementsByTagName( "ErrorMessage" )[0].InnerText,
                xml.GetElementsByTagName( "PaymentLog" )[0].InnerText,
                xml.GetElementsByTagName( "ReferenceID" )[0].InnerText,
                xml.GetElementsByTagName( "AvsAddrStatus" )[0].InnerText,
                xml.GetElementsByTagName( "AvsZipStatus" )[0].InnerText,
                xml.GetElementsByTagName( "CvvStatus" )[0].InnerText );
            return paymentResult;
        }

        private string CreateOnWebsitePaymentXml(
            Culture culture,
            decimal orderTotal,
            string orderID,
            string ipAddress,
            string storefrontUrl,
            string recurringXml,
            bool isLastProduct
            )
        {

            string notifyURL = String.Empty;
            string siteName = EncodeSpecialXmlCharacters( DataAccessContext.Configurations.GetValue( culture.CultureID, "SiteName" ) );
            string postData = "<?xml version=\"1.0\"?>" +
                    "<ChargeRequest>";

            postData += XMLAuthenticationBuilder();

            if (_checkoutDetails.PaymentMethod.Name == "PayPal Pro")
            {
                notifyURL = storefrontUrl + "Gateway/PayPalNotification.aspx";
            }

            PaymentSignatureServices paymentSignature = new PaymentSignatureServices();

            postData += "   <ChargeInformation>" +
                   "            <PaymentName>" + _checkoutDetails.PaymentMethod.Name + "</PaymentName>" +
                   "            <Token>" + _checkoutDetails.PaymentToken + "</Token>" +
                   "            <IPAddress>" + ipAddress + "</IPAddress>" +
                   "            <Amount>" + orderTotal + "</Amount>" +
                   "            <CurrencyCode>" + DataAccessContext.Configurations.GetValue( "PaymentCurrency" ) + "</CurrencyCode>" +
                   "            <SiteName>" + siteName + "</SiteName>" +
                   "            <OrderID>" + orderID + "</OrderID>" +
                   "            <NotifyUrl>" + notifyURL + "</NotifyUrl>" +
                   "            <IsLastProduct>" + ConvertUtilities.ToString( isLastProduct ) + "</IsLastProduct>" +
                   "            <VevoPaymentSignature>" + paymentSignature.CreateKey( _checkoutDetails.PaymentToken, orderTotal.ToString(), orderID ) +
                   "            </VevoPaymentSignature>" +
                   "        </ChargeInformation>";

            postData += recurringXml;

            postData += "</ChargeRequest>";

            return postData;
        }

        private static string AppendPath( string path1, string path2 )
        {
            if (path1.EndsWith( "/" ) && path2.StartsWith( "/" ))
                return path1 + path2.Substring( 1 );
            else if (!path1.EndsWith( "/" ) && !path2.StartsWith( "/" ))
                return path1 + "/" + path2;
            else
                return path1 + path2;
        }

        private static string RemoveHttp( string url )
        {
            string lowered = url.ToLower();
            if (lowered.StartsWith( "http://" ))
                return url.Substring( 7 );
            else if (lowered.StartsWith( "https://" ))
                return url.Substring( 8 );
            else
                return url;
        }

        private string EncodeSpecialXmlCharacters( string text )
        {
            text = text.Replace( "&", "&amp;" );
            text = text.Replace( "<", "&lt;" );
            text = text.Replace( ">", "&gt;" );
            text = text.Replace( "\"", "&quot;" );
            text = text.Replace( "'", "&apos;" );
            return text;
        }

        private string ChangeSpecialCharacterToUnderScore( string text )
        {
            text = text.Replace( "&", "_" );
            text = text.Replace( "<", "_" );
            text = text.Replace( ">", "_" );
            text = text.Replace( "\"", "_" );
            text = text.Replace( "'", "_" );
            return text;
        }

        #endregion


        #region Constructors

        public PaymentAppGateway( CheckoutDetails checkoutDetails )
        {
            _checkoutDetails = checkoutDetails;
        }

        #endregion


        #region Public Methods

        public string CreateHostedPaymentXml(
            Culture culture,
            Currency currency,
            Cart shoppingCart,
            string storefrontUrl,
            string orderID,
            decimal orderTotal,
            WholesaleStatus wholesaleStatus,
            string ipAddress )
        {
            string returnURL = String.Empty;
            string cancelURL = String.Empty;
            string notifyURL = String.Empty;

            string postData = "<?xml version=\"1.0\"?>" +
                    "<HostedRequest>";

            postData += XMLAuthenticationBuilder();
            postData += XMLShoppingCartBuilder( culture, currency, shoppingCart, wholesaleStatus );

            if (_checkoutDetails.PaymentMethod.Name == "PayPal")
            {
                returnURL = storefrontUrl + "Gateway/PayPalReturn.aspx?OrderID=" + orderID;
                cancelURL = storefrontUrl;
                notifyURL = storefrontUrl + "Gateway/PayPalNotification.aspx";
            }
            else if (_checkoutDetails.PaymentMethod.Name == "RBSWorldPay")
            {
                notifyURL = storefrontUrl + "Gateway/RBSWorldPayNotification.aspx";
            }

            PaymentSignatureServices paymentSignature = new PaymentSignatureServices();

            postData += "   <HostedInformation>" +
                        "       <BillingAddress>" +
                        "           <FirstName>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.FirstName ) + "</FirstName>" +
                        "           <LastName>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.LastName ) + "</LastName>" +
                        "           <Company>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.Company ) + "</Company>" +
                        "           <Address1>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.Address1 ) + "</Address1>" +
                        "           <Address2>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.Address2 ) + "</Address2>" +
                        "           <City>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.City ) + "</City>" +
                        "           <State>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.State ) + "</State>" +
                        "           <Zip>" + _checkoutDetails.BillingAddress.Zip + "</Zip>" +
                        "           <Country>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.Country ) + "</Country>" +
                        "           <Phone>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.Phone ) + "</Phone>" +
                        "           <Fax>" + EncodeSpecialXmlCharacters( _checkoutDetails.BillingAddress.Fax ) + "</Fax>" +
                        "       </BillingAddress>" +
                        "       <ShippingAddress>" +
                        "           <FirstName>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.FirstName ) + "</FirstName>" +
                        "           <LastName>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.LastName ) + "</LastName>" +
                        "           <Company>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.Company ) + "</Company>" +
                        "           <Address1>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.Address1 ) + "</Address1>" +
                        "           <Address2>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.Address2 ) + "</Address2>" +
                        "           <City>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.City ) + "</City>" +
                        "           <State>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.State ) + "</State>" +
                        "           <Zip>" + _checkoutDetails.ShippingAddress.Zip + "</Zip>" +
                        "           <Country>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.Country ) + "</Country>" +
                        "           <Phone>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.Phone ) + "</Phone>" +
                        "           <Fax>" + EncodeSpecialXmlCharacters( _checkoutDetails.ShippingAddress.Fax ) + "</Fax>" +
                        "       </ShippingAddress>" +
                        "       <Email>" + _checkoutDetails.Email + "</Email>" +
                        "       <PaymentName>" + _checkoutDetails.PaymentMethod.Name + "</PaymentName>" +
                        "       <Amount>" + orderTotal + "</Amount>" +
                        "       <InvoiceNumber>" + orderID + "</InvoiceNumber>" +
                        "       <CurrencyCode>" + DataAccessContext.Configurations.GetValue( "PaymentCurrency" ) + "</CurrencyCode>" +
                        "       <ItemName>Payment for " + EncodeSpecialXmlCharacters( DataAccessContext.Configurations.GetValue( culture.CultureID, "SiteName" ) ) +
                        "       </ItemName>" +
                        "       <IPAddress>" + ipAddress + "</IPAddress>" +
                        "       <VerifyCode></VerifyCode>" +
                        "       <ReturnUrl>" + returnURL + "</ReturnUrl>" +
                        "       <CancelUrl>" + cancelURL + "</CancelUrl>" +
                        "       <NotifyUrl>" + notifyURL + "</NotifyUrl>" +
                        "       <VevoPaymentSignature>" + paymentSignature.CreateKey( "", orderTotal.ToString(), orderID ) +
                        "       </VevoPaymentSignature>" +
                        "   </HostedInformation>" +
                        "</HostedRequest>";

            return postData;
        }

        public string CreateOnWebsitePaymentXml(
            Culture culture,
            decimal orderTotal,
            string orderID,
            string ipAddress,
            string storefrontUrl,
            bool isLastProduct
            )
        {
            return CreateOnWebsitePaymentXml( culture, orderTotal, orderID, ipAddress, storefrontUrl, "", isLastProduct );
        }

        public string CreateOnWebsiteRecurringPaymentXml(
            Culture culture,
            Currency currency,
            OrderAmountPackage orderAmountPackage,
            ICartItem cartItem,
            string ipAddress,
            string storefrontUrl,
            bool isLastProduct )
        {
            string xmlData = "<RecurringInformation>" +
                             "  <ProductName>" + EncodeSpecialXmlCharacters( cartItem.GetName( culture, currency ) ) + "</ProductName>" +
                             "  <RecurringInterval>" + cartItem.RecurringInterval + "</RecurringInterval>" +
                             "  <RecurringIntervalUnit>" + cartItem.RecurringIntervalUnit + "</RecurringIntervalUnit>" +
                             "  <RecurringNumberOfTrialCycles>" + cartItem.RecurringNumberOfTrialCycles + "</RecurringNumberOfTrialCycles>" +
                             "  <RecurringNumberOfRegularCycles>" + cartItem.RecurringNumberOfCycles + "</RecurringNumberOfRegularCycles>" +
                             "  <RecurringRegularTotal>" + orderAmountPackage.RecurringRegularAmount.Total + "</RecurringRegularTotal>" +
                             "  <RecurringTrialTotal>" + orderAmountPackage.RecurringTrialAmount.Total + "</RecurringTrialTotal>" +
                             "</RecurringInformation>";

            return CreateOnWebsitePaymentXml( culture, 0, String.Empty, ipAddress, storefrontUrl, xmlData, isLastProduct );
        }

        public PaymentAppResult PostCommand( IHttpService httpService, string xmlData, string storefrontUrl )
        {
            string chargeUrl = GetPaymentAppUrl( "/Charge.aspx", storefrontUrl ); ;

            string responseData = httpService.PostData( chargeUrl, xmlData, "text/xml" );
            XmlDocument xml = new XmlDocument();
            xml.LoadXml( responseData );
            return GeneratePaymentResult( xml );
        }


        public static string GetPaymentAppUrl( string path, string storefrontUrl )
        {
            string postBackUrl = String.Empty;

            string appendedPath = AppendPath( DataAccessContext.Configurations.GetValue( "PaymentAppUrl" ), path );

            if (appendedPath.StartsWith( "~/" ))
            {
                appendedPath = appendedPath.Substring( "~/".Length );
                appendedPath = AppendPath( RemoveHttp( storefrontUrl ), appendedPath );
            }

            if (DataAccessContext.Configurations.GetBoolValue( "PaymentSSLEnabled" ))
            {
                postBackUrl = "https://" + appendedPath;
            }
            else
            {
                postBackUrl = "http://" + appendedPath;
            }

            return postBackUrl;
        }

        #endregion

    }
}
