using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Shared.DataAccess;
using Vevo.Shared.DataAccess.Implementation;

namespace Vevo.Reports
{
    public class CustomerReportBuilder
    {
        private const int DayMonthConversionXAxisThreshold = 60;       // Number in days

        private string GetTopKeyword( string keyword )
        {
            if (keyword == "Quantity")
            {
                return "Count(OrderID) as NumberOfOrder, SUM" + CreateTotalSqlNoAs() + " AS Total";
            }
            else
            {
                return "SUM" + CreateTotalSqlNoAs() + " AS Total, Count(OrderID) as NumberOfOrder";
            }
        }

        private string CreateTotalSqlNoAs()
        {
            return " (Subtotal + Tax + ShippingCost + HandlingFee - CouponDiscount - GiftCertificate) ";
        }

        private string GetKeyword( string keyword )
        {
            if (keyword == "Quantity")
            {
                return "Count(OrderID) DESC";
            }
            else
            {
                return "SUM" + CreateTotalSqlNoAs() + " DESC";
            }
        }

        private string GetTopNumber( string numberOfResult )
        {
            if (numberOfResult == "All")
            {
                return "";
            }
            else
            {
                return "TOP " + numberOfResult;
            }
        }

        private string CreateSortString( string sortBy, string function )
        {
            string[] sort = sortBy.Split( ' ' );
            if (sort[0] == ("RegisterPeriod"))
            {
                if (function == "year")
                {
                    return "ORDER BY " + DbImplementation.Sql.CreateYearMonthFormat( "RegisterDate" ) + sort[1];
                }
                else if (function == "day")
                {
                    return "ORDER BY " + DbImplementation.Sql.CreateDateFormat( "RegisterDate" ) + sort[1];
                }
                else
                {
                    return "ORDER BY " + DbImplementation.Sql.CreateHourFormat( "RegisterDate" ) + sort[1];
                }
            }
            else
            {
                return "ORDER BY COUNT(UserName) " + sort[1];
            }
        }

        private DataTable GetCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate )
        {
            string sort;

            if (!String.IsNullOrEmpty( sortBy ))
            {
                sort = CreateSortString( sortBy, "day" );
            }
            else
            {
                sort = "";
            }

            ArrayList parameters = new ArrayList();

            string script = "SELECT " +
                DbImplementation.Sql.CreateDateFormat( "RegisterDate" ) + " as RegisterPeriod, " +
                "COUNT(UserName) as RegisterCustomer  " +
                "FROM Customer " +
                "WHERE " +
                DateFilterHelper.CreateFromDateNoAnd( startDate, parameters, "from", "RegisterDate" ) +
                DateFilterHelper.CreateToDateWithAnd( endDate, parameters, "to", "RegisterDate" ) +
                "GROUP BY " + DbImplementation.Sql.CreateDateFormat( "RegisterDate" ) + " " +
                sort;

            return DataAccess.ExecuteSelect( script, DataAccess.ConvertArrayListToDbParameters( parameters ) );
        }

        private DataTable GetYearCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate )
        {
            string sort;

            if (!String.IsNullOrEmpty( sortBy ))
            {
                sort = CreateSortString( sortBy, "year" );
            }
            else
            {
                sort = "";
            }

            ArrayList parameters = new ArrayList();

            string script = "SELECT " +
                DbImplementation.Sql.CreateMonthNumberFormat( "RegisterDate" ) + " as [MonthNumber], " +
                DbImplementation.Sql.CreateMonthNameFormat( "RegisterDate" ) + " as [MonthName], " +
                DbImplementation.Sql.CreateYearFormat( "RegisterDate" ) + " as [Year], " +
                DbImplementation.Sql.CreateMonthYearFormat( "RegisterDate" ) + " as RegisterPeriod, " +
                DbImplementation.Sql.CreateYearMonthFormat( "RegisterDate" ) + " as RegisterDate," +
                "       COUNT(UserName) as RegisterCustomer " +
                "FROM Customer " +
                "WHERE " +
                DateFilterHelper.CreateFromDateNoAnd( startDate, parameters, "from", "RegisterDate" ) +
                DateFilterHelper.CreateToDateWithAnd( endDate, parameters, "to", "RegisterDate" ) +
                "GROUP BY " +
                DbImplementation.Sql.CreateMonthNumberFormat( "RegisterDate" ) + " , " +
                DbImplementation.Sql.CreateMonthNameFormat( "RegisterDate" ) + " , " +
                DbImplementation.Sql.CreateYearFormat( "RegisterDate" ) + " , " +
                DbImplementation.Sql.CreateMonthYearFormat( "RegisterDate" ) + " , " +
                DbImplementation.Sql.CreateYearMonthFormat( "RegisterDate" ) + " " +
                sort;

            return DataAccess.ExecuteSelect( script, DataAccess.ConvertArrayListToDbParameters( parameters ) );
        }

        private DataTable GetOneDayReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate,
            string reportType )
        {
            string sort, from, to;

            if (!String.IsNullOrEmpty( sortBy ))
            {
                sort = CreateSortString( sortBy, "hour" );
            }
            else
            {
                sort = "";
            }

            ArrayList parameters = new ArrayList();

            if (reportType == "24Hours")
            {
                from = DateFilterHelper.CreateFromDateFor24HoursNoAnd(
                    startDate, parameters, "from", "RegisterDate" );
                to = DateFilterHelper.CreateToDateFor24HoursWithAnd(
                    endDate, parameters, "to", "RegisterDate" );
            }
            else
            {
                from = DateFilterHelper.CreateFromDateNoAnd( startDate, parameters, "from", "RegisterDate" );
                to = DateFilterHelper.CreateToDateWithAnd( endDate, parameters, "to", "RegisterDate" );
            }

            string script = "SELECT " +
                DbImplementation.Sql.CreateDateFormat( "RegisterDate" ) + " as RegisterDate, " +
                DbImplementation.Sql.CreateHourFormat( "RegisterDate" ) + " as RegisterPeriod, " +
                " Count(UserName) as RegisterCustomer " +
                "FROM Customer " +
                "WHERE " +
                from +
                to +
                "GROUP BY " +
                DbImplementation.Sql.CreateDateFormat( "RegisterDate" ) + " , " +
                DbImplementation.Sql.CreateHourFormat( "RegisterDate" ) + " " +
                sort;

            return DataAccess.ExecuteSelect( script, DataAccess.ConvertArrayListToDbParameters( parameters ) );
        }

        private DataTable Get24HoursCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate )
        {
            DataTable customerTable = GetOneDayReport( sortBy, startDate, endDate, "24Hours" );
            return customerTable;
        }

        private DataTable GetOneDayCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate )
        {
            DataTable customerTable = GetOneDayReport( sortBy, startDate, endDate, "OneDay" );
            return customerTable;
        }

        public DataTable GetCustomerReport( DateTime startDate, DateTime endDate )
        {
            return GetCustomerReport( "", startDate, endDate );
        }

        private DataTable GetCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {

            DataTable full = GetCustomerReport( sortBy, startDate, endDate );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["RegisterPeriod"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable GetYearCustomerReport( DateTime startDate, DateTime endDate )
        {
            return GetYearCustomerReport( "", startDate, endDate );
        }

        private DataTable GetYearCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {

            DataTable full = GetYearCustomerReport( sortBy, startDate, endDate );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["RegisterPeriod"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        private DataTable GetTopCustomerReport(
            string topKeyword,
            string numberOfResult,
            string sortBy,
            DateTime startDate,
            DateTime endDate,
            PeriodType period )
        {
            string from;
            string to;
            ArrayList parameters = new ArrayList();

            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( true ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );
            if (period == PeriodType.Last24Hours)
            {
                from = DateFilterHelper.CreateFromDateFor24HoursWithAnd(
                    startDate, parameters, "from", "OrderDate" );
                to = DateFilterHelper.CreateToDateFor24HoursWithAnd(
                    endDate, parameters, "to", "OrderDate" );
            }
            else
            {
                from = DateFilterHelper.CreateFromDateWithAnd( startDate, parameters, "from", "OrderDate" );
                to = DateFilterHelper.CreateToDateWithAnd( endDate, parameters, "to", "OrderDate" );
            }

            string sort = "";
            if (numberOfResult != "All")
            {
                sort = "ORDER BY " + GetKeyword( topKeyword );
            }

            string sort1 = "";
            if (sortBy.Contains( "Average" ))
            {
                string[] sortArray = sortBy.Split( ' ' );
                sort1 = "ORDER BY (Total/NumberOfOrder) " + sortArray[1];
            }
            else
            {
                sort1 = "ORDER BY " + sortBy;
            }

            string script = "SELECT Total, NumberOfOrder, (Total/NumberOfOrder) as Average, Customer.UserName, " +
                "Customer.FirstName, Customer.LastName, Customer.RegisterDate, Customer.Email, " +
                "Customer.WholeSaleLevel, Customer.Phone, Customer.CustomerID " +
                "FROM Customer " +
                "INNER JOIN (SELECT " + GetTopNumber( numberOfResult ) + " " + GetTopKeyword( topKeyword ) + " " +
                ", CustomerID " +
                "FROM Orders " +
                "WHERE PaymentComplete = @PaymentComplete " +
                "AND Processed = @Processed And Cancelled = @Cancelled  " +
                from +
                to +
                "GROUP BY CustomerID " +
                sort +
                ") AS O " +
                "ON Customer.CustomerID = O.CustomerID " +
                sort1;

            DataTable TopCustomerTable = DataAccess.ExecuteSelect( script, DataAccess.
                    ConvertArrayListToDbParameters( parameters ) );

            if (numberOfResult != "All")
            {
                int numresult = Convert.ToInt32( numberOfResult );
                if (sortBy.Contains( "ASC" ))
                {
                    if (TopCustomerTable.Rows.Count > numresult)
                    {
                        int numberOfRemove = TopCustomerTable.Rows.Count - numresult;
                        for (int i = 0; i < numberOfRemove; i++)
                            TopCustomerTable.Rows.RemoveAt( i );
                    }
                }
                else
                {
                    if (TopCustomerTable.Rows.Count > numresult)
                    {
                        int numberOfRemove = TopCustomerTable.Rows.Count - numresult;
                        for (int i = 0; i < numberOfRemove; i++)
                            TopCustomerTable.Rows.RemoveAt( numresult );
                    }
                }
            }

            return TopCustomerTable;
        }

        private DataTable GetTopCustomerReport(
            string topKeyword,
            string numberOfResult,
            string sortBy,
            DateTime startDate,
            DateTime endDate,
            int startIndex,
            int endIndex,
            PeriodType period,
            out int howManyItems )
        {
            DataTable full = GetTopCustomerReport(
                topKeyword,
                numberOfResult,
                sortBy,
                startDate,
                endDate,
                period );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["Customer.CustomerID"] };
            full.PrimaryKey = keys;

            DataView view = new DataView( full );
            view.Sort = sortBy;

            DataTable sortedTable = view.ToTable();

            return DataAccess.CopyTableRows( sortedTable, startIndex, endIndex );
        }

        public DataTable GetOneDayCustomerReport( DateTime startDate, DateTime endDate )
        {
            return GetOneDayCustomerReport( "", startDate, endDate );
        }

        private DataTable GetOneDayCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable full = GetOneDayCustomerReport( sortBy, startDate, endDate );
            howManyItems = full.Rows.Count;
            DataColumn[] keys = new DataColumn[] { full.Columns["RegisterPeriod"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable Get24HoursCustomerReport( DateTime startDate, DateTime endDate )
        {
            return Get24HoursCustomerReport( "", startDate, endDate );
        }

        private DataTable Get24HoursCustomerReport(
            string sortBy,
            DateTime startDate,
            DateTime endDate,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable full = Get24HoursCustomerReport( sortBy, startDate, endDate );
            howManyItems = full.Rows.Count;
            DataColumn[] keys = new DataColumn[] { full.Columns["RegisterPeriod"] };
            full.PrimaryKey = keys;
            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        public DataTable GetTopCustomerReportData(
            string sortBy,
            PeriodType period,
            CustomerReportType customerReportType,
            string numDisplay,
            DateTime startDate,
            DateTime endDate,
            int startIndex,
            int endIndex,
            out int totalItems )
        {
            string topKeyword = String.Empty;
            if (customerReportType == CustomerReportType.TopNumberOfProductBuyer)
            {
                topKeyword = "Quantity";
            }
            else if (customerReportType == CustomerReportType.TopProductTotalPriceBuyer ||
                customerReportType == CustomerReportType.UserRegistration)
            {
                topKeyword = "Total";
            }

            DataTable table = GetTopCustomerReport(
                topKeyword,
                numDisplay,
                sortBy,
                startDate,
                endDate,
                startIndex,
                endIndex,
                period,
                out totalItems );
            return table;
        }

        public DataTable GetCustomerReportData(
            string sortBy,
            PeriodType period,
            DateTime startDate,
            DateTime endDate,
            int startIndex,
            int endIndex,
            out int totlaItems )
        {
            DataTable table;
            if (period == PeriodType.Custom)
            {
                period = ReportFilterUtilities.ResolveCustomPeriodType( startDate, endDate );
            }

            if (period == PeriodType.Yesterday || period == PeriodType.Today)
            {
                table = GetOneDayCustomerReport( sortBy, startDate, endDate,
                            startIndex, endIndex, out totlaItems );
            }
            else if (period == PeriodType.Last24Hours)
            {
                table = Get24HoursCustomerReport( sortBy, startDate, endDate,
                            startIndex, endIndex, out totlaItems );
            }
            else if (period == PeriodType.ThisYear || period == PeriodType.Last3Years)
            {
                table = GetYearCustomerReport( sortBy, startDate, endDate,
                            startIndex, endIndex, out totlaItems );
            }
            else
            {
                table = GetCustomerReport( sortBy, startDate, endDate,
                        startIndex, endIndex, out totlaItems );
            }
            return table;
        }

        public DataTable GetCustomerReportData(
            PeriodType period,
            DateTime startDate,
            DateTime endDate )
        {
            DataTable table;

            if (period == PeriodType.Custom)
            {
                period = ReportFilterUtilities.ResolveCustomPeriodType( startDate, endDate );
            }

            if (period == PeriodType.Yesterday || period == PeriodType.Today)
            {
                table = GetOneDayCustomerReport( startDate, endDate );
            }
            else if (period == PeriodType.Last24Hours)
            {
                table = Get24HoursCustomerReport( startDate, endDate );
            }
            else if (period == PeriodType.ThisYear || period == PeriodType.Last3Years)
            {
                table = GetYearCustomerReport( startDate, endDate );
            }
            else
            {
                table = GetCustomerReport( startDate, endDate );
            }
            return table;
        }

        public DataTable GetTopCustomerReportData(
            string sortBy,
            PeriodType period,
            CustomerReportType customerReportType,
            string numDisplay,
            DateTime startDate,
            DateTime endDate
            )
        {
            string topKeyword = String.Empty;
            if (customerReportType == CustomerReportType.TopNumberOfProductBuyer)
            {
                topKeyword = "Quantity";
            }
            else if (customerReportType == CustomerReportType.TopProductTotalPriceBuyer ||
                customerReportType == CustomerReportType.UserRegistration)
            {
                topKeyword = "Total";
            }

            DataTable table;
            table = GetTopCustomerReport(
                topKeyword,
                numDisplay,
                sortBy,
                startDate,
                endDate,
                period );
            return table;
        }
    }
}
